@@ -205,12 +205,19 @@ LLAMA_ROPE_SCALING_LINEAR = 1
 LLAMA_ROPE_SCALING_YARN = 2
 LLAMA_ROPE_SCALING_MAX_VALUE = LLAMA_ROPE_SCALING_YARN
 
+
 # typedef struct llama_token_data {
 #     llama_token id; // token id
 #     float logit;    // log-odds of the token
 #     float p;        // probability of the token
 # } llama_token_data;
 class llama_token_data(Structure):
+    """Used to store token data
+    
+    Attributes:
+        id (llama_token): token id
+        logit (float): log-odds of the token
+        p (float): probability of the token"""
     _fields_ = [
         ("id", llama_token),
         ("logit", c_float),
@@ -227,6 +234,12 @@ llama_token_data_p = POINTER(llama_token_data)
 #     bool sorted;
 # } llama_token_data_array;
 class llama_token_data_array(Structure):
+    """Used to sample tokens given logits
+    
+    Attributes:
+        data (ctypes.Array[llama_token_data]): token data
+        size (int): size of the array
+        sorted (bool): whether the array is sorted"""
     _fields_ = [
         ("data", llama_token_data_p),
         ("size", c_size_t),
@@ -271,6 +284,18 @@ llama_progress_callback = ctypes.CFUNCTYPE(None, c_float, c_void_p)
 #     llama_seq_id all_seq_id; // used if seq_id == NULL
 # } llama_batch;
 class llama_batch(Structure):
+    """Input data for llama_decode
+
+    A llama_batch object can contain input about one or many sequences
+
+    The provided arrays (i.e. token, embd, pos, etc.) must have size of n_tokens
+
+    Attributes:
+        token (ctypes.Array[llama_token]): the token ids of the input (used when embd is NULL)
+        embd (ctypes.Array[ctypes.c_float]): token embeddings (i.e. float vector of size n_embd) (used when token is NULL)
+        pos (ctypes.Array[ctypes.Array[llama_pos]]): the positions of the respective token in the sequence
+        seq_id (ctypes.Array[ctypes.Array[llama_seq_id]]): the sequence to which the respective token belongs"""
+
     _fields_ = [
         ("n_tokens", c_int32),
         ("token", POINTER(llama_token)),
@@ -302,6 +327,17 @@ class llama_batch(Structure):
 #     bool use_mlock;  // force system to keep model in RAM
 # };
 class llama_model_params(Structure):
+    """Parameters for llama_model
+    
+    Attributes:
+        n_gpu_layers (int): number of layers to store in VRAM
+        main_gpu (int): the GPU that is used for scratch and small tensors
+        tensor_split (ctypes.Array[ctypes.c_float]): how to split layers across multiple GPUs (size: LLAMA_MAX_DEVICES)
+        progress_callback (llama_progress_callback): called with a progress value between 0 and 1, pass NULL to disable
+        progress_callback_user_data (ctypes.c_void_p): context pointer passed to the progress callback
+        vocab_only (bool): only load the vocabulary, no weights
+        use_mmap (bool): use mmap if possible
+        use_mlock (bool): force system to keep model in RAM"""
     _fields_ = [
         ("n_gpu_layers", c_int32),
         ("main_gpu", c_int32),
@@ -325,7 +361,7 @@ class llama_model_params(Structure):
 #     // ref: https://github.com/ggerganov/llama.cpp/pull/2054
 #     float    rope_freq_base;   // RoPE base frequency, 0 = from model
 #     float    rope_freq_scale;  // RoPE frequency scaling factor, 0 = from model
-#     float    yarn_ext_factor;  // YaRN extrapolation mix factor, NaN = from model
+#     float    yarn_ext_factor;  // YaRN extrapolation mix factor, negative = from model
 #     float    yarn_attn_factor; // YaRN magnitude scaling factor
 #     float    yarn_beta_fast;   // YaRN low correction dim
 #     float    yarn_beta_slow;   // YaRN high correction dim
@@ -339,6 +375,26 @@ class llama_model_params(Structure):
 #     bool embedding;  // embedding mode only
 # };
 class llama_context_params(Structure):
+    """Parameters for llama_context
+    
+    Attributes:
+        seed (int): RNG seed, -1 for random
+        n_ctx (int): text context, 0 = from model
+        n_batch (int): prompt processing maximum batch size
+        n_threads (int): number of threads to use for generation
+        n_threads_batch (int): number of threads to use for batch processing
+        rope_scaling_type (int): RoPE scaling type, from `enum llama_rope_scaling_type`
+        rope_freq_base (float): RoPE base frequency, 0 = from model
+        rope_freq_scale (float): RoPE frequency scaling factor, 0 = from model
+        yarn_ext_factor (float): YaRN extrapolation mix factor, negative = from model
+        yarn_attn_factor (float): YaRN magnitude scaling factor
+        yarn_beta_fast (float): YaRN low correction dim
+        yarn_beta_slow (float): YaRN high correction dim
+        yarn_orig_ctx (int): YaRN original context size
+        mul_mat_q (bool): if true, use experimental mul_mat_q kernels (DEPRECATED - always true)
+        f16_kv (bool): use fp16 for KV cache, fp32 otherwise
+        logits_all (bool): the llama_eval() call computes all logits, not just the last one
+        embedding (bool): embedding mode only"""
     _fields_ = [
         ("seed", c_uint32),
         ("n_ctx", c_uint32),
@@ -367,6 +423,11 @@ class llama_context_params(Structure):
 # // It might not exist for progress report where '.' is output repeatedly.
 # typedef void (*llama_log_callback)(enum llama_log_level level, const char * text, void * user_data);
 llama_log_callback = ctypes.CFUNCTYPE(None, c_int, c_char_p, c_void_p)
+"""Signature for logging events
+Note that text includes the new line character at the end for most events.
+If your logging mechanism cannot handle that, check if the last character is '\n' and strip it
+if it exists.
+It might not exist for progress report where '.' is output repeatedly."""
 
 
 # // model quantization parameters
@@ -379,6 +440,15 @@ llama_log_callback = ctypes.CFUNCTYPE(None, c_int, c_char_p, c_void_p)
 #     bool pure;                   // disable k-quant mixtures and quantize all tensors to the same type
 # } llama_model_quantize_params;
 class llama_model_quantize_params(Structure):
+    """Parameters for llama_model_quantize
+    
+    Attributes:
+        nthread (int): number of threads to use for quantizing, if <=0 will use std::thread::hardware_concurrency()
+        ftype (int): quantize to this llama_ftype
+        allow_requantize (bool): allow quantizing non-f32/f16 tensors
+        quantize_output_tensor (bool): quantize output.weight
+        only_copy (bool): only copy tensors - ftype, allow_requantize and quantize_output_tensor are ignored
+        pure (bool): disable k-quant mixtures and quantize all tensors to the same type"""
     _fields_ = [
         ("nthread", c_int),
         ("ftype", c_int),
@@ -470,6 +540,7 @@ class llama_timings(Structure):
 # // Helpers for getting default parameters
 # LLAMA_API struct llama_model_params llama_model_default_params(void);
 def llama_model_default_params() -> llama_model_params:
+    """Get default parameters for llama_model"""
     return _lib.llama_model_default_params()
 
 
@@ -479,6 +550,7 @@ _lib.llama_model_default_params.restype = llama_model_params
 
 # LLAMA_API struct llama_context_params llama_context_default_params(void);
 def llama_context_default_params() -> llama_context_params:
+    """Get default parameters for llama_context"""
     return _lib.llama_context_default_params()
 
 
@@ -488,6 +560,7 @@ _lib.llama_context_default_params.restype = llama_context_params
 
 # LLAMA_API struct llama_model_quantize_params llama_model_quantize_default_params(void);
 def llama_model_quantize_default_params() -> llama_model_quantize_params:
+    """Get default parameters for llama_model_quantize"""
     return _lib.llama_model_quantize_default_params()
 
 
@@ -500,6 +573,9 @@ _lib.llama_model_quantize_default_params.restype = llama_model_quantize_params
 # // Call once at the start of the program
 # LLAMA_API void llama_backend_init(bool numa);
 def llama_backend_init(numa: Union[c_bool, bool]):
+    """Initialize the llama + ggml backend
+    If numa is true, use NUMA optimizations
+    Call once at the start of the program"""
     return _lib.llama_backend_init(numa)
 
 
@@ -510,6 +586,7 @@ _lib.llama_backend_init.restype = None
 # // Call once at the end of the program - currently only used for MPI
 # LLAMA_API void llama_backend_free(void);
 def llama_backend_free():
+    """Call once at the end of the program - currently only used for MPI"""
     return _lib.llama_backend_free()
 
 
@@ -555,6 +632,7 @@ _lib.llama_new_context_with_model.restype = llama_context_p
 # // Frees all allocated memory
 # LLAMA_API void llama_free(struct llama_context * ctx);
 def llama_free(ctx: llama_context_p):
+    """Frees all allocated memory"""
     return _lib.llama_free(ctx)
 
 
@@ -655,18 +733,80 @@ _lib.llama_n_embd.restype = c_int
 # // Get the model's RoPE frequency scaling factor
 # LLAMA_API float llama_rope_freq_scale_train(const struct llama_model * model);
 def llama_rope_freq_scale_train(model: llama_model_p) -> float:
+    """Get the model's RoPE frequency scaling factor"""
     return _lib.llama_rope_freq_scale_train(model)
 
 
 _lib.llama_rope_freq_scale_train.argtypes = [llama_model_p]
 _lib.llama_rope_freq_scale_train.restype = c_float
 
+# // Functions to access the model's GGUF metadata scalar values
+# // - The functions return the length of the string on success, or -1 on failure
+# // - The output string is always null-terminated and cleared on failure
+# // - GGUF array values are not supported by these functions
+
+
+# // Get metadata value as a string by key name
+# LLAMA_API int llama_model_meta_val_str(const struct llama_model * model, const char * key, char * buf, size_t buf_size);
+def llama_model_meta_val_str(
+    model: llama_model_p, key: Union[c_char_p, bytes], buf: bytes, buf_size: int
+) -> int:
+    """Get metadata value as a string by key name"""
+    return _lib.llama_model_meta_val_str(model, key, buf, buf_size)
+
+
+_lib.llama_model_meta_val_str.argtypes = [llama_model_p, c_char_p, c_char_p, c_size_t]
+_lib.llama_model_meta_val_str.restype = c_int
+
+
+# // Get the number of metadata key/value pairs
+# LLAMA_API int llama_model_meta_count(const struct llama_model * model);
+def llama_model_meta_count(model: llama_model_p) -> int:
+    """Get the number of metadata key/value pairs"""
+    return _lib.llama_model_meta_count(model)
+
+
+_lib.llama_model_meta_count.argtypes = [llama_model_p]
+_lib.llama_model_meta_count.restype = c_int
+
+
+# // Get metadata key name by index
+# LLAMA_API int llama_model_meta_key_by_index(const struct llama_model * model, int i, char * buf, size_t buf_size);
+def llama_model_meta_key_by_index(
+    model: llama_model_p, i: Union[c_int, int], buf: bytes, buf_size: int
+) -> int:
+    """Get metadata key name by index"""
+    return _lib.llama_model_meta_key_by_index(model, i, buf, buf_size)
+
+
+_lib.llama_model_meta_key_by_index.argtypes = [llama_model_p, c_int, c_char_p, c_size_t]
+_lib.llama_model_meta_key_by_index.restype = c_int
+
+
+# // Get metadata value as a string by index
+# LLAMA_API int llama_model_meta_val_str_by_index(const struct llama_model * model, int i, char * buf, size_t buf_size);
+def llama_model_meta_val_str_by_index(
+    model: llama_model_p, i: Union[c_int, int], buf: bytes, buf_size: int
+) -> int:
+    """Get metadata value as a string by index"""
+    return _lib.llama_model_meta_val_str_by_index(model, i, buf, buf_size)
+
+
+_lib.llama_model_meta_val_str_by_index.argtypes = [
+    llama_model_p,
+    c_int,
+    c_char_p,
+    c_size_t,
+]
+_lib.llama_model_meta_val_str_by_index.restype = c_int
+
 
 # // Get a string describing the model type
 # LLAMA_API int llama_model_desc(const struct llama_model * model, char * buf, size_t buf_size);
 def llama_model_desc(
     model: llama_model_p, buf: bytes, buf_size: Union[c_size_t, int]
 ) -> int:
+    """Get a string describing the model type"""
     return _lib.llama_model_desc(model, buf, buf_size)
 
 
@@ -677,6 +817,7 @@ _lib.llama_model_desc.restype = c_int
 # // Returns the total size of all the tensors in the model in bytes
 # LLAMA_API uint64_t llama_model_size(const struct llama_model * model);
 def llama_model_size(model: llama_model_p) -> int:
+    """Returns the total size of all the tensors in the model in bytes"""
     return _lib.llama_model_size(model)
 
 
@@ -687,6 +828,7 @@ _lib.llama_model_size.restype = ctypes.c_uint64
 # // Returns the total number of parameters in the model
 # LLAMA_API uint64_t llama_model_n_params(const struct llama_model * model);
 def llama_model_n_params(model: llama_model_p) -> int:
+    """Returns the total number of parameters in the model"""
     return _lib.llama_model_n_params(model)
 
 
@@ -699,6 +841,7 @@ _lib.llama_model_n_params.restype = ctypes.c_uint64
 def llama_get_model_tensor(
     model: llama_model_p, name: Union[c_char_p, bytes]
 ) -> c_void_p:
+    """Get a llama model tensor"""
     return _lib.llama_get_model_tensor(model, name)
 
 
@@ -716,6 +859,7 @@ def llama_model_quantize(
     fname_out: bytes,
     params,  # type: POINTER(llama_model_quantize_params) # type: ignore
 ) -> int:
+    """Returns 0 on success"""
     return _lib.llama_model_quantize(fname_inp, fname_out, params)
 
 
@@ -747,6 +891,12 @@ def llama_apply_lora_from_file(
     path_base_model: Union[c_char_p, bytes],
     n_threads: Union[c_int, int],
 ) -> int:
+    """Apply a LoRA adapter to a loaded model
+    path_base_model is the path to a higher quality model to use as a base for
+    the layers modified by the adapter. Can be NULL to use the current loaded model.
+    The model needs to be reloaded before applying a new adapter, otherwise the adapter
+    will be applied on top of the previous one
+    Returns 0 on success"""
     return _lib.llama_apply_lora_from_file(
         ctx, path_lora, scale, path_base_model, n_threads
     )
@@ -794,10 +944,103 @@ _lib.llama_model_apply_lora_from_file.restype = c_int
 # //
 
 
-# // Returns the number of tokens in the KV cache
-# LLAMA_API DEPRECATED(int llama_get_kv_cache_token_count(const struct llama_context * ctx),
-#         "avoid using this, it will be removed in the future, instead - count the tokens in user code");
+# // Information associated with an individual cell in the KV cache view.
+# struct llama_kv_cache_view_cell {
+#     // The position for this cell. Takes KV cache shifts into account.
+#     // May be negative if the cell is not populated.
+#     llama_pos pos;
+# };
+class llama_kv_cache_view_cell(Structure):
+    _fields_ = [("pos", llama_pos)]
+
+
+# // An updateable view of the KV cache.
+# struct llama_kv_cache_view {
+#     // Number of KV cache cells. This will be the same as the context size.
+#     int32_t n_cells;
+
+#     // Maximum number of sequences that can exist in a cell. It's not an error
+#     // if there are more sequences in a cell than this value, however they will
+#     // not be visible in the view cells_sequences.
+#     int32_t n_max_seq;
+
+#     // Number of tokens in the cache. For example, if there are two populated
+#     // cells, the first with 1 sequence id in it and the second with 2 sequence
+#     // ids then you'll have 3 tokens.
+#     int32_t token_count;
+
+#     // Number of populated cache cells.
+#     int32_t used_cells;
+
+#     // Maximum contiguous empty slots in the cache.
+#     int32_t max_contiguous;
+
+#     // Index to the start of the max_contiguous slot range. Can be negative
+#     // when cache is full.
+#     int32_t max_contiguous_idx;
+
+#     // Information for an individual cell.
+#     struct llama_kv_cache_view_cell * cells;
+
+
+#     // The sequences for each cell. There will be n_max_seq items per cell.
+#     llama_seq_id * cells_sequences;
+# };
+class llama_kv_cache_view(Structure):
+    _fields_ = [
+        ("n_cells", c_int32),
+        ("n_max_seq", c_int32),
+        ("token_count", c_int32),
+        ("used_cells", c_int32),
+        ("max_contiguous", c_int32),
+        ("max_contiguous_idx", c_int32),
+        ("cells", POINTER(llama_kv_cache_view_cell)),
+        ("cells_sequences", POINTER(llama_seq_id)),
+    ]
+
+
+# // Create an empty KV cache view. (use only for debugging purposes)
+# LLAMA_API struct llama_kv_cache_view llama_kv_cache_view_init(const struct llama_context * ctx, int32_t n_max_seq);
+def llama_kv_cache_view_init(
+    ctx: llama_context_p, n_max_seq: Union[c_int32, int]
+) -> llama_kv_cache_view:
+    """Create an empty KV cache view. (use only for debugging purposes)"""
+    return _lib.llama_kv_cache_view_init(ctx, n_max_seq)
+
+
+_lib.llama_kv_cache_view_init.argtypes = [llama_context_p, c_int32]
+_lib.llama_kv_cache_view_init.restype = llama_kv_cache_view
+
+
+# // Free a KV cache view. (use only for debugging purposes)
+# LLAMA_API void llama_kv_cache_view_free(struct llama_kv_cache_view * view);
+def llama_kv_cache_view_free(view: llama_kv_cache_view):
+    """Free a KV cache view. (use only for debugging purposes)"""
+    return _lib.llama_kv_cache_view_free(view)
+
+
+_lib.llama_kv_cache_view_free.argtypes = [llama_kv_cache_view]
+_lib.llama_kv_cache_view_free.restype = None
+
+
+# // Update the KV cache view structure with the current state of the KV cache. (use only for debugging purposes)
+# LLAMA_API void llama_kv_cache_view_update(const struct llama_context * ctx, struct llama_kv_cache_view * view);
+def llama_kv_cache_view_update(ctx: llama_context_p, view: llama_kv_cache_view):
+    """Update the KV cache view structure with the current state of the KV cache. (use only for debugging purposes)"""
+    return _lib.llama_kv_cache_view_update(ctx, view)
+
+
+_lib.llama_kv_cache_view_update.argtypes = [llama_context_p, llama_kv_cache_view]
+_lib.llama_kv_cache_view_update.restype = None
+
+
+# // Returns the number of tokens in the KV cache (slow, use only for debug)
+# // If a KV cell has multiple sequences assigned to it, it will be counted multiple times
+# LLAMA_API int llama_get_kv_cache_token_count(const struct llama_context * ctx);
 def llama_get_kv_cache_token_count(ctx: llama_context_p) -> int:
+    """Returns the number of tokens in the KV cache (slow, use only for debug)
+    If a KV cell has multiple sequences assigned to it, it will be counted multiple times
+    """
     return _lib.llama_get_kv_cache_token_count(ctx)
 
 
@@ -805,10 +1048,22 @@ _lib.llama_get_kv_cache_token_count.argtypes = [llama_context_p]
 _lib.llama_get_kv_cache_token_count.restype = c_int
 
 
+# // Returns the number of used KV cells (i.e. have at least one sequence assigned to them)
+# LLAMA_API int llama_get_kv_cache_used_cells(const struct llama_context * ctx);
+def llama_get_kv_cache_used_cells(ctx: llama_context_p) -> int:
+    """Returns the number of used KV cells (i.e. have at least one sequence assigned to them)"""
+    return _lib.llama_get_kv_cache_used_cells(ctx)
+
+
+_lib.llama_get_kv_cache_used_cells.argtypes = [llama_context_p]
+_lib.llama_get_kv_cache_used_cells.restype = c_int
+
+
 # // Clear the KV cache
 # LLAMA_API void llama_kv_cache_clear(
 #         struct llama_context * ctx);
 def llama_kv_cache_clear(ctx: llama_context_p):
+    """Clear the KV cache"""
     return _lib.llama_kv_cache_clear(ctx)
 
 
@@ -831,6 +1086,10 @@ def llama_kv_cache_seq_rm(
     p0: Union[llama_pos, int],
     p1: Union[llama_pos, int],
 ):
+    """Removes all tokens that belong to the specified sequence and have positions in [p0, p1)
+    seq_id < 0 : match any sequence
+    p0 < 0     : [0,  p1]
+    p1 < 0     : [p0, inf)"""
     return _lib.llama_kv_cache_seq_rm(ctx, seq_id, p0, p1)
 
 
@@ -860,6 +1119,10 @@ def llama_kv_cache_seq_cp(
     p0: Union[llama_pos, int],
     p1: Union[llama_pos, int],
 ):
+    """Copy all tokens that belong to the specified sequence to another sequence
+    Note that this does not allocate extra KV cache memory - it simply assigns the tokens to the new sequence
+    p0 < 0 : [0,  p1]
+    p1 < 0 : [p0, inf)"""
     return _lib.llama_kv_cache_seq_cp(ctx, seq_id_src, seq_id_dst, p0, p1)
 
 
@@ -881,6 +1144,7 @@ def llama_kv_cache_seq_keep(
     ctx: llama_context_p,
     seq_id: Union[llama_seq_id, int],
 ):
+    """Removes all tokens that do not belong to the specified sequence"""
     return _lib.llama_kv_cache_seq_keep(ctx, seq_id)
 
 
@@ -905,6 +1169,10 @@ def llama_kv_cache_seq_shift(
     p1: Union[llama_pos, int],
     delta: Union[llama_pos, int],
 ):
+    """Adds relative position "delta" to all tokens that belong to the specified sequence and have positions in [p0, p1)
+    If the KV cache is RoPEd, the KV data is updated accordingly
+    p0 < 0 : [0,  p1]
+    p1 < 0 : [p0, inf)"""
     return _lib.llama_kv_cache_seq_shift(ctx, seq_id, p0, p1, delta)
 
 
@@ -926,6 +1194,8 @@ _lib.llama_kv_cache_seq_shift.restype = None
 # and kv_cache) - will often be smaller after compacting tokens
 # LLAMA_API size_t llama_get_state_size(const struct llama_context * ctx);
 def llama_get_state_size(ctx: llama_context_p) -> int:
+    """Returns the maximum size in bytes of the state (rng, logits, embedding
+    and kv_cache) - will often be smaller after compacting tokens"""
     return _lib.llama_get_state_size(ctx)
 
 
@@ -942,6 +1212,9 @@ _lib.llama_get_state_size.restype = c_size_t
 def llama_copy_state_data(
     ctx: llama_context_p, dst  # type: Array[c_uint8]
 ) -> int:
+    """Copies the state to the specified destination address.
+    Destination needs to have allocated enough memory.
+    Returns the number of bytes copied"""
     return _lib.llama_copy_state_data(ctx, dst)
 
 
@@ -957,6 +1230,7 @@ _lib.llama_copy_state_data.restype = c_size_t
 def llama_set_state_data(
     ctx: llama_context_p, src  # type: Array[c_uint8]
 ) -> int:
+    """Set the state reading from the specified address"""
     return _lib.llama_set_state_data(ctx, src)
 
 
@@ -1037,6 +1311,11 @@ def llama_eval(
     n_tokens: Union[c_int, int],
     n_past: Union[c_int, int],
 ) -> int:
+    """Run the llama inference to obtain the logits and probabilities for the next token(s).
+    tokens + n_tokens is the provided batch of new tokens to process
+    n_past is the number of tokens to use from previous eval calls
+    Returns 0 on success
+    DEPRECATED: use llama_decode() instead"""
     return _lib.llama_eval(ctx, tokens, n_tokens, n_past)
 
 
@@ -1058,6 +1337,8 @@ def llama_eval_embd(
     n_tokens: Union[c_int, int],
     n_past: Union[c_int, int],
 ) -> int:
+    """Same as llama_eval, but use float matrix input directly.
+    DEPRECATED: use llama_decode() instead"""
     return _lib.llama_eval_embd(ctx, embd, n_tokens, n_past)
 
 
@@ -1080,6 +1361,10 @@ def llama_batch_get_one(
     pos_0: Union[llama_pos, int],
     seq_id: llama_seq_id,
 ) -> llama_batch:
+    """Return batch for single sequence of tokens starting at pos_0
+
+    NOTE: this is a helper function to facilitate transition to the new batch API - avoid using it
+    """
     return _lib.llama_batch_get_one(tokens, n_tokens, pos_0, seq_id)
 
 
@@ -1108,6 +1393,13 @@ def llama_batch_init(
     embd: Union[c_int32, int],
     n_seq_max: Union[c_int32, int],
 ) -> llama_batch:
+    """Allocates a batch of tokens on the heap that can hold a maximum of n_tokens
+    Each token can be assigned up to n_seq_max sequence ids
+    The batch has to be freed with llama_batch_free()
+    If embd != 0, llama_batch.embd will be allocated with size of n_tokens * embd * sizeof(float)
+    Otherwise, llama_batch.token will be allocated to store n_tokens llama_token
+    The rest of the llama_batch members are allocated with size n_tokens
+    All members are left uninitialized"""
     return _lib.llama_batch_init(n_tokens, embd, n_seq_max)
 
 
@@ -1118,6 +1410,7 @@ _lib.llama_batch_init.restype = llama_batch
 # // Frees a batch of tokens allocated with llama_batch_init()
 # LLAMA_API void llama_batch_free(struct llama_batch batch);
 def llama_batch_free(batch: llama_batch):
+    """Frees a batch of tokens allocated with llama_batch_init()"""
     return _lib.llama_batch_free(batch)
 
 
@@ -1133,6 +1426,10 @@ _lib.llama_batch_free.restype = None
 #         struct llama_context * ctx,
 #           struct llama_batch   batch);
 def llama_decode(ctx: llama_context_p, batch: llama_batch) -> int:
+    """Positive return values does not mean a fatal error, but rather a warning.
+    0 - success
+    1 - could not find a KV slot for the batch (try reducing the size of the batch or increase the context)
+    < 0 - error"""
     return _lib.llama_decode(ctx, batch)
 
 
@@ -1149,6 +1446,10 @@ def llama_set_n_threads(
     n_threads: Union[c_uint32, int],
     n_threads_batch: Union[c_uint32, int],
 ):
+    """Set the number of threads used for decoding
+    n_threads is the number of threads used for generation (single token)
+    n_threads_batch is the number of threads used for prompt and batch processing (multiple tokens)
+    """
     return _lib.llama_set_n_threads(ctx, n_threads, n_threads_batch)
 
 
@@ -1165,6 +1466,11 @@ _lib.llama_set_n_threads.restype = None
 def llama_get_logits(
     ctx: llama_context_p,
 ):  # type: (...) -> Array[float] # type: ignore
+    """Token logits obtained from the last call to llama_eval()
+    The logits for the last token are stored in the last row
+    Logits for which llama_batch.logits[i] == 0 are undefined
+    Rows: n_tokens provided with llama_batch
+    Cols: n_vocab"""
     return _lib.llama_get_logits(ctx)
 
 
@@ -1178,6 +1484,8 @@ _lib.llama_get_logits.restype = c_float_p
 def llama_get_logits_ith(
     ctx: llama_context_p, i: Union[c_int32, int]
 ):  # type: (...) -> Array[float] # type: ignore
+    """Logits for the ith token. Equivalent to:
+    llama_get_logits(ctx) + i*n_vocab"""
     return _lib.llama_get_logits_ith(ctx, i)
 
 
@@ -1191,6 +1499,8 @@ _lib.llama_get_logits_ith.restype = c_float_p
 def llama_get_embeddings(
     ctx: llama_context_p,
 ):  # type: (...) -> Array[float] # type: ignore
+    """Get the embeddings for the input
+    shape: [n_embd] (1-dimensional)"""
     return _lib.llama_get_embeddings(ctx)
 
 
@@ -1213,7 +1523,9 @@ _lib.llama_token_get_text.restype = c_char_p
 
 
 # LLAMA_API float llama_token_get_score(const struct llama_model * model, llama_token token);
-def llama_token_get_score(model: llama_model_p, token: Union[llama_token, int]) -> float:
+def llama_token_get_score(
+    model: llama_model_p, token: Union[llama_token, int]
+) -> float:
     return _lib.llama_token_get_score(model, token)
 
 
@@ -1235,6 +1547,7 @@ _lib.llama_token_get_type.restype = ctypes.c_int
 
 # LLAMA_API llama_token llama_token_bos(const struct llama_model * model); // beginning-of-sentence
 def llama_token_bos(model: llama_model_p) -> int:
+    """beginning-of-sentence"""
     return _lib.llama_token_bos(model)
 
 
@@ -1244,6 +1557,7 @@ _lib.llama_token_bos.restype = llama_token
 
 # LLAMA_API llama_token llama_token_eos(const struct llama_model * model); // end-of-sentence
 def llama_token_eos(model: llama_model_p) -> int:
+    """end-of-sentence"""
     return _lib.llama_token_eos(model)
 
 
@@ -1253,6 +1567,7 @@ _lib.llama_token_eos.restype = llama_token
 
 # LLAMA_API llama_token llama_token_nl (const struct llama_model * model); // next-line
 def llama_token_nl(model: llama_model_p) -> int:
+    """next-line"""
     return _lib.llama_token_nl(model)
 
 
@@ -1260,9 +1575,32 @@ _lib.llama_token_nl.argtypes = [llama_model_p]
 _lib.llama_token_nl.restype = llama_token
 
 
+# // Returns -1 if unknown, 1 for true or 0 for false.
+# LLAMA_API int         llama_add_bos_token(const struct llama_model * model);
+def llama_add_bos_token(model: llama_model_p) -> int:
+    """Returns -1 if unknown, 1 for true or 0 for false."""
+    return _lib.llama_add_bos_token(model)
+
+
+_lib.llama_add_bos_token.argtypes = [llama_model_p]
+_lib.llama_add_bos_token.restype = c_int
+
+
+# // Returns -1 if unknown, 1 for true or 0 for false.
+# LLAMA_API int         llama_add_eos_token(const struct llama_model * model);
+def llama_add_eos_token(model: llama_model_p) -> int:
+    """Returns -1 if unknown, 1 for true or 0 for false."""
+    return _lib.llama_add_eos_token(model)
+
+
+_lib.llama_add_eos_token.argtypes = [llama_model_p]
+_lib.llama_add_eos_token.restype = c_int
+
+
 # // codellama infill tokens
 # LLAMA_API llama_token llama_token_prefix(const struct llama_model * model); // Beginning of infill prefix
 def llama_token_prefix(model: llama_model_p) -> int:
+    """codellama infill tokens"""
     return _lib.llama_token_prefix(model)
 
 
@@ -1325,6 +1663,7 @@ def llama_tokenize(
     add_bos: Union[c_bool, bool],
     special: Union[c_bool, bool],
 ) -> int:
+    """Convert the provided text into tokens."""
     return _lib.llama_tokenize(
         model, text, text_len, tokens, n_max_tokens, add_bos, special
     )
@@ -1357,6 +1696,11 @@ def llama_token_to_piece(
     buf: Union[c_char_p, bytes],
     length: Union[c_int, int],
 ) -> int:
+    """Token Id -> Piece.
+    Uses the vocabulary in the provided context.
+    Does not write null terminator to the buffer.
+    User code is responsible to remove the leading whitespace of the first non-BOS token when decoding multiple tokens.
+    """
     return _lib.llama_token_to_piece(model, token, buf, length)
 
 
@@ -1378,6 +1722,7 @@ def llama_grammar_init(
     n_rules: Union[c_size_t, int],
     start_rule_index: Union[c_size_t, int],
 ) -> llama_grammar_p:
+    """Initialize a grammar from a set of rules."""
     return _lib.llama_grammar_init(rules, n_rules, start_rule_index)
 
 
@@ -1391,6 +1736,7 @@ _lib.llama_grammar_init.restype = llama_grammar_p
 
 # LLAMA_API void llama_grammar_free(struct llama_grammar * grammar);
 def llama_grammar_free(grammar: llama_grammar_p):
+    """Free a grammar."""
     return _lib.llama_grammar_free(grammar)
 
 
@@ -1400,6 +1746,7 @@ _lib.llama_grammar_free.restype = None
 
 # LLAMA_API struct llama_grammar * llama_grammar_copy(const struct llama_grammar * grammar);
 def llama_grammar_copy(grammar: llama_grammar_p) -> llama_grammar_p:
+    """Copy a grammar."""
     return _lib.llama_grammar_copy(grammar)
 
 
@@ -1414,6 +1761,7 @@ _lib.llama_grammar_copy.restype = llama_grammar_p
 # // Sets the current rng seed.
 # LLAMA_API void llama_set_rng_seed(struct llama_context * ctx, uint32_t seed);
 def llama_set_rng_seed(ctx: llama_context_p, seed: Union[c_uint32, int]):
+    """Sets the current rng seed."""
     return _lib.llama_set_rng_seed(ctx, seed)
 
 
@@ -1440,6 +1788,9 @@ def llama_sample_repetition_penalties(
     penalty_freq: Union[c_float, float],
     penalty_present: Union[c_float, float],
 ):
+    """Repetition penalty described in CTRL academic paper https://arxiv.org/abs/1909.05858, with negative logit fix.
+    Frequency and presence penalties described in OpenAI API https://platform.openai.com/docs/api-reference/parameter-details.
+    """
     return _lib.llama_sample_repetition_penalties(
         ctx,
         candidates,
@@ -1478,6 +1829,7 @@ def llama_sample_classifier_free_guidance(
     guidance_ctx: llama_context_p,
     scale: Union[c_float, float],
 ):
+    """Apply classifier-free guidance to the logits as described in academic paper "Stay on topic with Classifier-Free Guidance" https://arxiv.org/abs/2306.17806"""
     return _lib.llama_sample_classifier_free_guidance(
         ctx, candidates, guidance_ctx, scale
     )
@@ -1499,6 +1851,7 @@ _lib.llama_sample_classifier_free_guidance.restype = None
 def llama_sample_softmax(
     ctx: llama_context_p, candidates  # type: _Pointer[llama_token_data]
 ):
+    """Sorts candidate tokens by their logits in descending order and calculate probabilities based on logits."""
     return _lib.llama_sample_softmax(ctx, candidates)
 
 
@@ -1521,6 +1874,7 @@ def llama_sample_top_k(
     k: Union[c_int, int],
     min_keep: Union[c_size_t, int],
 ):
+    """Top-K sampling described in academic paper "The Curious Case of Neural Text Degeneration" https://arxiv.org/abs/1904.09751"""
     return _lib.llama_sample_top_k(ctx, candidates, k, min_keep)
 
 
@@ -1545,6 +1899,7 @@ def llama_sample_top_p(
     p: Union[c_float, float],
     min_keep: Union[c_size_t, int],
 ):
+    """Nucleus sampling described in academic paper "The Curious Case of Neural Text Degeneration" https://arxiv.org/abs/1904.09751"""
     return _lib.llama_sample_top_p(ctx, candidates, p, min_keep)
 
 
@@ -1569,6 +1924,7 @@ def llama_sample_min_p(
     p: Union[c_float, float],
     min_keep: Union[c_size_t, int],
 ):
+    """Minimum P sampling as described in https://github.com/ggerganov/llama.cpp/pull/3841"""
     return _lib.llama_sample_min_p(ctx, candidates, p, min_keep)
 
 
@@ -1593,6 +1949,7 @@ def llama_sample_tail_free(
     z: Union[c_float, float],
     min_keep: Union[c_size_t, int],
 ):
+    """Tail Free Sampling described in https://www.trentonbricken.com/Tail-Free-Sampling/."""
     return _lib.llama_sample_tail_free(ctx, candidates, z, min_keep)
 
 
@@ -1617,6 +1974,7 @@ def llama_sample_typical(
     p: Union[c_float, float],
     min_keep: Union[c_size_t, int],
 ):
+    """Locally Typical Sampling implementation described in the paper https://arxiv.org/abs/2202.00666."""
     return _lib.llama_sample_typical(ctx, candidates, p, min_keep)
 
 
@@ -1638,6 +1996,11 @@ def llama_sample_temp(
     candidates,  # type: _Pointer[llama_token_data_array]
     temp: Union[c_float, float],
 ):
+    """Temperature sampling described in academic paper "Generating Long Sequences with Sparse Transformers" https://arxiv.org/abs/1904.10509
+    
+    Parameters:
+        candidates: A vector of `llama_token_data` containing the candidate tokens, their probabilities (p), and log-odds (logit) for the current position in the generated text.
+        temp: The temperature value to use for the sampling. A higher value corresponds to more surprising or less predictable text, while a lower value corresponds to less surprising or more predictable text."""
     return _lib.llama_sample_temp(ctx, candidates, temp)
 
 
@@ -1659,6 +2022,7 @@ def llama_sample_temperature(
     candidates,  # type: _Pointer[llama_token_data_array]
     temp: Union[c_float, float],
 ):
+    """use llama_sample_temp instead"""
     return _lib.llama_sample_temperature(ctx, candidates, temp)
 
 
@@ -1680,6 +2044,11 @@ def llama_sample_grammar(
     candidates,  # type: _Pointer[llama_token_data_array]
     grammar,  # type: llama_grammar_p
 ):
+    """Apply constraints from grammar
+    
+    Parameters:
+        candidates: A vector of `llama_token_data` containing the candidate tokens, their probabilities (p), and log-odds (logit) for the current position in the generated text.
+        grammar: A grammar object containing the rules and constraints to apply to the generated text."""
     return _lib.llama_sample_grammar(ctx, candidates, grammar)
 
 
@@ -1712,6 +2081,14 @@ def llama_sample_token_mirostat(
     m: Union[c_int, int],
     mu,  # type: _Pointer[c_float]
 ) -> int:
+    """Mirostat 1.0 algorithm described in the paper https://arxiv.org/abs/2007.14966. Uses tokens instead of words.
+    
+    Parameters:
+        candidates: A vector of `llama_token_data` containing the candidate tokens, their probabilities (p), and log-odds (logit) for the current position in the generated text.
+        tau: The target cross-entropy (or surprise) value you want to achieve for the generated text. A higher value corresponds to more surprising or less predictable text, while a lower value corresponds to less surprising or more predictable text.
+        eta: The learning rate used to update `mu` based on the error between the target and observed surprisal of the sampled word. A larger learning rate will cause `mu` to be updated more quickly, while a smaller learning rate will result in slower updates.
+        m: The number of tokens considered in the estimation of `s_hat`. This is an arbitrary value that is used to calculate `s_hat`, which in turn helps to calculate the value of `k`. In the paper, they use `m = 100`, but you can experiment with different values to see how it affects the performance of the algorithm.
+        mu: Maximum cross-entropy. This value is initialized to be twice the target cross-entropy (`2 * tau`) and is updated in the algorithm based on the error between the target and observed surprisal."""
     return _lib.llama_sample_token_mirostat(ctx, candidates, tau, eta, m, mu)
 
 
@@ -1744,6 +2121,13 @@ def llama_sample_token_mirostat_v2(
     eta: Union[c_float, float],
     mu,  # type: _Pointer[c_float]
 ) -> int:
+    """Mirostat 2.0 algorithm described in the paper https://arxiv.org/abs/2007.14966. Uses tokens instead of words.
+    
+    Parameters:
+        candidates: A vector of `llama_token_data` containing the candidate tokens, their probabilities (p), and log-odds (logit) for the current position in the generated text.
+        tau: The target cross-entropy (or surprise) value you want to achieve for the generated text. A higher value corresponds to more surprising or less predictable text, while a lower value corresponds to less surprising or more predictable text.
+        eta: The learning rate used to update `mu` based on the error between the target and observed surprisal of the sampled word. A larger learning rate will cause `mu` to be updated more quickly, while a smaller learning rate will result in slower updates.
+        mu: Maximum cross-entropy. This value is initialized to be twice the target cross-entropy (`2 * tau`) and is updated in the algorithm based on the error between the target and observed surprisal."""
     return _lib.llama_sample_token_mirostat_v2(ctx, candidates, tau, eta, mu)
 
 
@@ -1766,6 +2150,7 @@ def llama_sample_token_greedy(
     ctx: llama_context_p,
     candidates,  # type: _Pointer[llama_token_data_array]
 ) -> int:
+    """Selects the token with the highest probability."""
     return _lib.llama_sample_token_greedy(ctx, candidates)
 
 
@@ -1784,6 +2169,7 @@ def llama_sample_token(
     ctx: llama_context_p,
     candidates,  # type: _Pointer[llama_token_data_array]
 ) -> int:
+    """Randomly selects a token from the candidates based on their probabilities."""
     return _lib.llama_sample_token(ctx, candidates)
 
 
@@ -1804,6 +2190,7 @@ def llama_grammar_accept_token(
     grammar: llama_grammar_p,
     token: Union[llama_token, int],
 ) -> None:
+    """Accepts the sampled token into the grammar"""
     _lib.llama_grammar_accept_token(ctx, grammar, token)
 
 
@@ -1906,6 +2293,7 @@ _lib.llama_beam_search.restype = None
 
 # LLAMA_API struct llama_timings llama_get_timings(struct llama_context * ctx);
 def llama_get_timings(ctx: llama_context_p) -> llama_timings:
+    """Get performance information"""
     return _lib.llama_get_timings(ctx)
 
 
@@ -1915,6 +2303,7 @@ _lib.llama_get_timings.restype = llama_timings
 
 # LLAMA_API void llama_print_timings(struct llama_context * ctx);
 def llama_print_timings(ctx: llama_context_p):
+    """Print performance information"""
     _lib.llama_print_timings(ctx)
 
 
@@ -1924,6 +2313,7 @@ _lib.llama_print_timings.restype = None
 
 # LLAMA_API void llama_reset_timings(struct llama_context * ctx);
 def llama_reset_timings(ctx: llama_context_p):
+    """Reset performance information"""
     _lib.llama_reset_timings(ctx)
 
 
@@ -1934,6 +2324,7 @@ _lib.llama_reset_timings.restype = None
 # Print system information
 # LLAMA_API const char * llama_print_system_info(void);
 def llama_print_system_info() -> bytes:
+    """Print system information"""
     return _lib.llama_print_system_info()
 
 
@@ -1948,6 +2339,9 @@ _lib.llama_print_system_info.restype = c_char_p
 def llama_log_set(
     log_callback: "ctypes._FuncPointer", user_data: c_void_p  # type: ignore
 ):
+    """Set callback for all future logging events.
+
+    If this is not called, or NULL is supplied, everything is output on stderr."""
     return _lib.llama_log_set(log_callback, user_data)
 
 
